using Asp.Versioning;
using Luski.Shared.PublicServers.V1.ClientToServer.HTTP;
using Luski.Shared.PublicServers.V1.Enums;
using Luski.Shared.PublicServers.V1.ServerToClient.HTTP;
using Luski.Shared.PublicServers.V1.Shared;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using Microsoft.AspNetCore.Mvc;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class SocketCategoryController : ControllerBase
{
    [HttpPost]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Post([FromBody]CategoryPostCTS ChanReq)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!LuskiFunctions.HasAccessToCategory(ID, ChanReq.Parent, out ServerPermission opt, ServerPermission.CreateCategories)) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            DateTime epoch = LuskiFunctions.Config.ServerEpoch;
            List<long> RoleOverides = new(), UserOverides = new();
            if (ChanReq.RoleOverrides.Length > 0)
            {
                foreach (UserRoleOverrideCTS over in ChanReq.RoleOverrides)
                {
                    LuskiFunctions.Snowflake overSnowflake = LuskiFunctions.Snowflake.GenerateSnowflake(epoch);
                    Tables.ServerRoleOverrides.Insert(
                        ServerRoleOverrides.ID.CreateParameter(overSnowflake.ID),
                        ServerRoleOverrides.RoleID.CreateParameter(over.RoleID),
                        ServerRoleOverrides.GoodPermissions.CreateParameter(over.GoodPermissions),
                        ServerRoleOverrides.BadPermissions.CreateParameter(over.BadPermissions));
                    RoleOverides.Add(overSnowflake.ID);
                }
            }
            if (ChanReq.UserOverrides.Length > 0)
            {
                foreach (UserOverrideCTS over in ChanReq.UserOverrides)
                {
                    LuskiFunctions.Snowflake overSnowflake = LuskiFunctions.Snowflake.GenerateSnowflake(epoch);
                    Tables.UserRoleOverrides.Insert(
                        UserRoleOverrides.ID.CreateParameter(overSnowflake.ID),
                        UserRoleOverrides.UserID.CreateParameter(over.UserID),
                        UserRoleOverrides.GoodPermissions.CreateParameter(over.GoodPermissions),
                        UserRoleOverrides.BadPermissions.CreateParameter(over.BadPermissions));
                    UserOverides.Add(overSnowflake.ID);
                }
            }

            epoch = DateTime.UtcNow.Date;
            LuskiFunctions.Snowflake chanSnowflake = LuskiFunctions.Snowflake.GenerateSnowflake(epoch);
            Tables.Categories.Insert(
                Categories.ID.CreateParameter(chanSnowflake.ID),
                Categories.Parent.CreateParameter(ChanReq.Parent),
                Categories.Name.CreateParameter(Convert.FromBase64String(ChanReq.Name)),
                Categories.Description.CreateParameter(Convert.FromBase64String(ChanReq.Description)),
                Categories.RoleOverrides.CreateParameter(RoleOverides.ToArray()),
                Categories.UserOverrides.CreateParameter(UserOverides.ToArray()),
                Categories.TitleEncryptionKey.CreateParameter(ChanReq.TitleEncryptionKey),
                Categories.DescriptionEncryptionKey.CreateParameter(ChanReq.DescriptionEncryptionKey),
                Categories.TitleEncoderType.CreateParameter(ChanReq.TitleEncoderType),
                Categories.DescriptionEncoderType.CreateParameter(ChanReq.DescriptionEncoderType));
            CategoryRow chan = Tables.Categories.ReadRow(Categories.ID.CreateParameter(chanSnowflake.ID));
            if (opt.HasPermission(ServerPermission.ViewCategories))
            {
                chan.InnerCategories = Array.Empty<long>();
            }
            if (opt.HasPermission(ServerPermission.ViewChannels))
            {
                chan.Channels = Array.Empty<long>();
            }
            return this.ResponseToResult(new CategorySTC()
            {
                Name = Convert.ToBase64String(chan.Name),
                Description = Convert.ToBase64String(chan.Description),
                ID = chanSnowflake.ID,
                Channels = chan.Channels,
                InnerCategories = chan.InnerCategories,
                Parent = chan.Parent,
                TitleEncryptionKey = chan.TitleEncryptionKey,
                DescriptionEncryptionKey = chan.DescriptionEncryptionKey,
                TitleEncoderType = chan.TitleEncoderType,
                DescriptionEncoderType = chan.DescriptionEncoderType,
                UserOverrides = chan.UserOverrides,
                RoleOverrides = chan.RoleOverrides,
                Color = Convert.ToHexString(chan.Color)
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpDelete]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Delete([FromHeader(Name = "id")]long Channel)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!LuskiFunctions.HasAccessToCategory(ID, Channel, ServerPermission.DeleteChannels)) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            Tables.Channels.DeleteRow(Channels.ID.CreateParameter(Channel));
            Tables.Messages.DeleteRow(Messages.ChannelID.CreateParameter(Channel));
            return StatusCode(StatusCodes.Status202Accepted);
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpPatch]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Patch([FromHeader(Name = "id")]long Channel)  //TODO Add Channel edit latter
    {
        return null!;
    }
    
    [HttpGet]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Get([FromHeader(Name = "id")]long Channel)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!LuskiFunctions.HasAccessToCategory(ID, Channel, out ServerPermission opt, ServerPermission.ViewCategories | ServerPermission.ViewChannels)) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            CategoryRow chan = Tables.Categories.ReadRow(Categories.ID.CreateParameter(Channel));
            if (opt.HasPermission(ServerPermission.ViewCategories))
            {
                chan.InnerCategories = Array.Empty<long>();
            }
            if (opt.HasPermission(ServerPermission.ViewChannels))
            {
                chan.Channels = Array.Empty<long>();
            }
            return this.ResponseToResult(new CategorySTC()
            {
                Name = Convert.ToBase64String(chan.Name),
                Description = Convert.ToBase64String(chan.Description),
                ID = Channel,
                Channels = chan.Channels,
                InnerCategories = chan.InnerCategories,
                Parent = chan.Parent,
                TitleEncryptionKey = chan.TitleEncryptionKey,
                DescriptionEncryptionKey = chan.DescriptionEncryptionKey,
                TitleEncoderType = chan.TitleEncoderType,
                DescriptionEncoderType = chan.DescriptionEncoderType,
                UserOverrides = chan.UserOverrides,
                RoleOverrides = chan.RoleOverrides,
                Color = Convert.ToHexString(chan.Color)
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
}