using System.Net.Mime;
using Asp.Versioning;
using Luski.Shared.PublicServers.V1.ClientToServer.HTTP;
using Luski.Shared.PublicServers.V1.Enums;
using Luski.Shared.PublicServers.V1.ServerToClient.HTTP;
using LuskiServer.Classes;
using LuskiServer.Classes.ActionFilters;
using LuskiServer.Classes.TableDef;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.ModelBinding;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class SocketUserProfileController : ControllerBase
{
    /// <summary>
    /// Returns the Status of a user.
    /// </summary>
    /// <param name="id">The Id of the user</param>
    /// <returns></returns>
    [HttpGet]
    [ProducesResponseType(StatusCodes.Status200OK)]
    [ProducesResponseType(typeof(STC), StatusCodes.Status403Forbidden, MediaTypeNames.Application.Json)]
    [Produces(MediaTypeNames.Text.Plain, MediaTypeNames.Application.Json)]
    [Route(LuskiFunctions.Info.Routes.Default.Base + "/Status/{id:long}")]
    public UserStatus Status(long id)
    {
        UserStatus status = Tables.Users.Read(Users.Status, Users.ID.CreateParameter(id));
        return status;
    }

    /// <summary>
    /// Returns the Avatar for the user.
    /// </summary>
    /// <param name="id">The ID of the requested user</param>
    /// <returns></returns>
    [HttpGet]
    [ProducesResponseType(StatusCodes.Status200OK)]
    [ProducesResponseType(typeof(STC), StatusCodes.Status403Forbidden)]
    [Produces(MediaTypeNames.Image.Jpeg, MediaTypeNames.Image.Gif, MediaTypeNames.Image.Tiff, "image/png", MediaTypeNames.Application.Json)]
    [Route(LuskiFunctions.Info.Routes.Default.Base + "/Avatar/{id:long}")]
    public IActionResult? Avatar(long id)
    {
        if (Tables.Users.TryRead(Users.Picture, out byte[]? image, Users.ID.CreateParameter(id)))
        {
            return Tables.Users.Read(Users.PictureType, Users.ID.CreateParameter(id)) switch
            {
                PictureType.png => File(image, "image/png"),
                PictureType.jpeg => File(image, "image/jpeg"),
                PictureType.bmp => File(image, "image/bmp"),
                PictureType.gif => File(image, "image/gif"),
                PictureType.ico => File(image, "image/vnd.microsoft.icon"),
                PictureType.svg => File(image, "image/svg+xml"),
                PictureType.tif => File(image, "image/tiff"),
                PictureType.webp => File(image, "image/webp"),
                //should never happen
                _ => File(image, "image/png"),
            };
        }
        else
        {
            return StatusCode(403, new STC()
            {
                Error = ErrorCode.Forbidden,
                ErrorMessage = "the user you have givven does not exist"
            });
            //return File(Encoding.UTF8.GetBytes(obj.ToString()), "application/json");
        }
    }
    
    /// <summary>
    /// Make a post to this endpoint to change the status of you account.
    /// </summary>
    /// <param name="SentData">Infomation required for status update.</param>
    /// <param name="token">The Token for your account.</param>
    /// <returns></returns>
    [HttpPost]
    [ProducesResponseType(StatusCodes.Status202Accepted)]
    [ProducesResponseType(typeof(STC), StatusCodes.Status403Forbidden)]
    [Produces(MediaTypeNames.Application.Json)]
    [Consumes(typeof(StatusUpdateCTS), MediaTypeNames.Application.Json)]
    [Route(LuskiFunctions.Info.Routes.Default.Base + "/Status")]
    [TokenFilter]
    public IActionResult Status([FromBody]StatusUpdateCTS SentData)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;

            //dynamic? SentData = Newtonsoft.Json.JsonConvert.DeserializeObject<dynamic>(content);
            UserStatus NewStatus = SentData.Status;
            UserStatus OldStatus = Tables.Users.Read(Users.Status, Users.ID.CreateParameter(ID));
            if (OldStatus != NewStatus && (int)NewStatus < (int)UserStatus.MAX && (int)NewStatus >= 0)
            {
               // JObject @out = new()
               // {
                  //  { "id", ID },
                  //  { "before", (int)OldStatus },
                  //  { "after", (int)NewStatus },
                  //  { "type", (int)DataType.Status_Update }
               // };
                Tables.Users.Update(Users.ID, ID, Users.Status.CreateParameter(NewStatus));
                if (NewStatus == UserStatus.Invisible) NewStatus = UserStatus.Offline;
               // WSS.SendData(SendType.All, @out);
            }

            return StatusCode(202);
        }
        catch (Exception ex)
        {
            return this.ShowError(ex);
        }
    }
/*
    [HttpPost]
    [Route(Luski.Info.Route.Version1.Action)]
    public async Task<IActionResult> Activity()
    {
        try
        {
            if (!this.CanTokenRequest(out long token_id, out HTTPResponse? toc) && toc != null) return StatusCode(403, toc);
            string content;

            using (StreamReader reader = new(Request.Body))
            {
                content = reader.ReadToEnd();
            }
            if (string.IsNullOrEmpty(content)) return StatusCode(403, this.ShowError<HTTPResponse>(ErrorCode.MissingPostData));
            dynamic? SentData = Newtonsoft.Json.JsonConvert.DeserializeObject<dynamic>(content);
            if (SentData == null) return this.ShowError<HTTPResponse>(ErrorCode.MissingPostData);
            if (string.IsNullOrEmpty((string)SentData.activity)) return this.ShowError<HTTPResponse>(ErrorCode.MissingPostData);
            string NewActivity = (string)SentData.activity;
            string OldActivity = Luski.Database.Read<string>("users", "activity", Luski.Database.CreateParameter("id", token_id));
            if (OldActivity != NewActivity)
            {
                JObject @out = new()
                {
                    { "id", token_id },
                    { "before", OldActivity },
                    { "after", NewActivity }
                };
                Luski.Database.Update("users", "id", token_id, Luski.Database.CreateParameter("activity", NewActivity));
                WSS.SendData(SendType.All, Program.Output(DataType.Status_Update, @out));
            }

            Response.Clear();
            Response.StatusCode = StatusCodes.Status200OK;
            await Response.StartAsync();
            return null;
        }
        catch (Exception ex)
        {
            return this.ShowError<HTTPResponse>(ErrorCode.ServerError, ex);
        }
    }*/
}